/* Copyright (c) 2000-2009 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/**
 * A regular expression hyperlink handler.
 *
 * @class
 * @base ko.hyperlinks.BaseHandler
 * @param name {string} A unique name of the hyperlink handler.
 * @param findRegex {regexp} Regular expression used to find the hyperlink.
 * @param fn {function} Will be called when the user clicks on the jump
 *        point. The matching (or replaced) hyperlink value is the only
 *        argument passed to this function.
 * @param replace_str {string|null} Optional - If set, will be used to
 *        change the matching text into a different value, similar to
 *        str.replace(findRegex, replace_str), so you can use the JavaScript
 *        regular expression positions: $1, $2, etc... in the string.
 * @param lang_names {array|null} Optional - If set, the handler will
 *        only show hyperlinks when the editor language is one of these
 *        language names.
 * @param indic_style {int} Optional - Indicator style, see scimoz.INDIC_*
 * @param indic_color {int} Optional - Indicator color (BGR), i.e. 0xFFCC33
 */
ko.hyperlinks.RegexHandler = function(name, findRegex, fn, replace_str, lang_names,
                                      indic_style, indic_color)
{
    var base_args = [name, fn, lang_names, indic_style, indic_color];
    ko.hyperlinks.BaseHandler.apply(this, base_args);
    this.findRegex = findRegex;
    this.replace_str = replace_str;
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
ko.hyperlinks.RegexHandler.prototype = new ko.hyperlinks.BaseHandler();
ko.hyperlinks.RegexHandler.prototype.constructor = ko.hyperlinks.RegexHandler;

/**
 * Try and show a hyperlink at the current position in the view.
 *
 * @param view {Components.interfaces.koIScintillaView}  View to check.
 * @param scimoz {Components.interfaces.ISciMoz}  Scimoz for the view.
 * @param position {int}  Position in the scimoz editor.
 * @param line {string}  The current line from the editor.
 * @param lineStartPos {int} Scimoz position for the start of the line.
 * @param lineEndPos {int}   Scimoz position for the end of the line.
 * @returns {ko.hyperlinks.Hyperlink} - The hyperlink instance shown.
 */
ko.hyperlinks.RegexHandler.prototype.show = function(view, scimoz, position, line,
                                            lineStartPos, lineEndPos)
{
    var match = this.findRegex.exec(line);
    var start = lineStartPos;
    var end;
    while (match) {
        //dump("RegexHandler:: '" + this.name + "' matched: " + match[0] + "\n");
        // The match must overlap the given position.
        start += ko.stringutils.bytelength(line.substr(0, match.index));
        if (start > position) {
            break;
        }
        end = start + ko.stringutils.bytelength(match[0]);
        if (end >= position) {
            var arg = match[0];
            if (this.replace_str) {
                arg = arg.replace(this.findRegex, this.replace_str);
            }
            //dump('Function arg: ' + arg + '\n');
            return this.setHyperlink(view, start, end, [arg]);
        }
        // Move past this match in the line, we'll check for
        // another match with on the rest of the line.
        line = line.substr(match.index + match[0].length);
        start = end;
        match = this.findRegex.exec(line);
    }
    return null;
}

// Add some basic hyperlink handlers.

// XXX: These regex handlers need to become Komodo preference settings.

ko.hyperlinks.addHandler(
    new ko.hyperlinks.RegexHandler(
        "Hyperlinks",
        new RegExp("(https?|ftps?):[^'\">\\s]+", "i"),
        ko.browse.openUrlInDefaultBrowser,
        null,  /* Use the found string instead of a replacement. */
        null,  /* All language types */
        Components.interfaces.ISciMoz.INDIC_PLAIN,
        RGB(0x60,0x90,0xff))
);

ko.hyperlinks.addHandler(
    new ko.hyperlinks.RegexHandler(
        "ActiveState bugs",
        new RegExp("bug\\s+(\\d{4,5})", "i"),
        ko.browse.openUrlInDefaultBrowser,
        "http://bugs.activestate.com/show_bug.cgi?id=$1",
        null   /* All language types */,
        Components.interfaces.ISciMoz.INDIC_PLAIN,
        RGB(0x60,0x90,0xff))
);
